﻿#Region "-- Imports --"
Imports Infragistics.Shared
Imports Infragistics.Win
Imports Infragistics.Win.UltraWinChart
Imports Infragistics.Win.UltraWinListView
Imports Infragistics.UltraChart.Core.Primitives
Imports Infragistics.UltraChart.Core
Imports Infragistics.UltraChart.Core.Layers
Imports Infragistics.UltraChart.Data
Imports Infragistics.UltraChart.Core.Util
Imports Infragistics.UltraChart.Resources.Appearance
Imports Infragistics.UltraChart.Shared.Styles
Imports Infragistics.Win.UltraWinToolTip

#End Region

Public Class Form1

#Region "-- Enumerations --"
  Public Enum PrimitiveListColumnKey As Short
    Type
    X
    Y
    W
    H
    GanttSeries
    GanttItem
  End Enum
  Public Enum EventListColumnKey As Short
    Time
    Name
  End Enum
#End Region

#Region "-- Local Variables --"
  Private _CustomPrimitives As New List(Of CustomPrimitive)
  Private _LineColor As Color = Color.Red
  Private _LineStyle As New LineStyle(LineCapStyle.Flat, LineCapStyle.Flat, LineDrawStyle.Solid)
  Private _ToolTipAppearance As New Infragistics.Win.Appearance()
#End Region

#Region "-- Event Handlers --"
#Region "  -- Form Events --"
  Private Sub Form1_Load(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Load
    SetupControls()
  End Sub
  Private Sub Form1_Shown(sender As Object, e As System.EventArgs) Handles Me.Shown
    CreateChartData()
  End Sub
#End Region
#Region "  -- UltraChart Events --"
  Private Sub UltraChart1_FillSceneGraph(ByVal sender As Object, ByVal e As Infragistics.UltraChart.Shared.Events.FillSceneGraphEventArgs) Handles UltraChart1.FillSceneGraph
    If Not Me.IsHandleCreated Then Exit Sub
    Dim i As Integer = 0
    ulvwPrimitives.Items.Clear()
    For Each o As Primitive In e.SceneGraph
      If o IsNot Nothing Then
        If TypeOf o Is Box Then
          Dim newItem As New UltraListViewItem(i.ToString)
          newItem.Value = i
          ulvwPrimitives.Items.Add(newItem)
          newItem.SubItems.Item(PrimitiveListColumnKey.Type.ToString).Value = o.GetType.Name
          If o.Series IsNot Nothing Then
            newItem.SubItems.Item(PrimitiveListColumnKey.GanttSeries.ToString).Value = o.Series.Key
          End If
          If o.DataPoint IsNot Nothing AndAlso TypeOf o.DataPoint Is GanttItem Then
            newItem.SubItems.Item(PrimitiveListColumnKey.GanttItem.ToString).Value = CType(o.DataPoint, GanttItem).Key
          End If
          Dim bx As Box = CType(o, Box)
          newItem.SubItems.Item(PrimitiveListColumnKey.X.ToString).Value = bx.rect.X
          newItem.SubItems.Item(PrimitiveListColumnKey.Y.ToString).Value = bx.rect.Y
          newItem.SubItems.Item(PrimitiveListColumnKey.W.ToString).Value = bx.rect.Width
          newItem.SubItems.Item(PrimitiveListColumnKey.H.ToString).Value = bx.rect.Height
        End If
      End If
      i += 1
    Next

    For Each customPrimitive As CustomPrimitive In _CustomPrimitives
      Dim newBox As New Box(New Point(customPrimitive.X, customPrimitive.Y), customPrimitive.Width, customPrimitive.Height, _LineStyle)
      newBox.lineStyle = _LineStyle
      newBox.PE.Fill = _LineColor
      newBox.PE.FillOpacity = 200
      newBox.PE.StrokeWidth = 1
      newBox.PE.StrokeOpacity = 255
      newBox.PE.Stroke = _LineColor
      newBox.Caps = PCaps.HitTest
      e.SceneGraph.Add(newBox)
    Next
  End Sub
  Private Sub UltraChart1_DataItemOut(sender As Object, e As Infragistics.UltraChart.Shared.Events.ChartDataEventArgs) Handles UltraChart1.DataItemOut
    Me.AddEvent("DataItemOut")
    Me.UltraToolTipManager1.HideToolTip()
    Me.UltraToolTipManager1.SetUltraToolTip(UltraChart1, Nothing)
  End Sub
  Private Sub UltraChart1_DataItemOver(sender As Object, e As Infragistics.UltraChart.Shared.Events.ChartDataEventArgs) Handles UltraChart1.DataItemOver
    Me.AddEvent("DataItemOver")
    ShowItemToolTip(e.Primitive.Series.Key, e.ColumnLabel, e.Primitive.TimeEntryIndex)
  End Sub
  Private Sub UltraChart1_MouseEnter(sender As Object, e As System.EventArgs) Handles UltraChart1.MouseEnter
    Me.AddEvent("MouseEnter")
  End Sub
  Private Sub UltraChart1_MouseHover(sender As Object, e As System.EventArgs) Handles UltraChart1.MouseHover
    Me.AddEvent("MouseHover")
  End Sub
  Private Sub UltraChart1_MouseLeave(sender As Object, e As System.EventArgs) Handles UltraChart1.MouseLeave
    Me.AddEvent("MouseLeave")
  End Sub
#End Region
#Region "  -- Other Events --"
  Private Sub btnClose_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnClose.Click
    Me.Close()
    End
  End Sub
  Private Sub btnFill_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnFill.Click
    Try
      CreateChartData()
    Catch ex As Exception
      MessageBox.Show(ex.InnerException.Message, String.Format("{0}.Fill Error", Me.GetType.Name), MessageBoxButtons.OK, MessageBoxIcon.Error)
    End Try
  End Sub
  Private Sub btnZoomLeftIn_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnZoomLeftIn.Click
    UltraChart1.Axis.X.RangeMin = (New Date(CLng(UltraChart1.Axis.X.RangeMin))).AddHours(1).Ticks
  End Sub
  Private Sub btnZoomLeftOut_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnZoomLeftOut.Click
    UltraChart1.Axis.X.RangeMin = (New Date(CLng(UltraChart1.Axis.X.RangeMin))).AddHours(-1).Ticks
  End Sub
  Private Sub btnZoomRightOut_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnZoomRightOut.Click
    UltraChart1.Axis.X.RangeMax = (New Date(CLng(UltraChart1.Axis.X.RangeMax))).AddHours(1).Ticks
  End Sub
  Private Sub btnZoomRightIn_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnZoomRightIn.Click
    UltraChart1.Axis.X.RangeMax = (New Date(CLng(UltraChart1.Axis.X.RangeMax))).AddHours(-1).Ticks
  End Sub
  Private Sub UltraScrollBar1_Scroll(ByVal sender As System.Object, ByVal e As System.Windows.Forms.ScrollEventArgs) Handles UltraScrollBar1.Scroll
    If UltraScrollBar1.Minimum <= e.NewValue And e.NewValue <= UltraScrollBar1.Maximum Then
      Dim startDateBefore As DateTime = (New Date(CLng(UltraChart1.Axis.X.RangeMin)))
      Dim endDateBefore As DateTime = (New Date(CLng(UltraChart1.Axis.X.RangeMax)))
      Dim spanTicks As Double = UltraChart1.Axis.X.RangeMax - UltraChart1.Axis.X.RangeMin

      Select Case e.Type
        Case ScrollEventType.LargeDecrement : startDateBefore = startDateBefore.AddDays(CShort(e.NewValue - e.OldValue))
        Case ScrollEventType.LargeIncrement : startDateBefore = startDateBefore.AddDays(CShort(e.NewValue - e.OldValue))
        Case ScrollEventType.SmallDecrement : startDateBefore = startDateBefore.AddHours(CShort(e.NewValue - e.OldValue))
        Case ScrollEventType.SmallIncrement : startDateBefore = startDateBefore.AddHours(CShort(e.NewValue - e.OldValue))
        Case ScrollEventType.ThumbTrack : startDateBefore = startDateBefore.AddHours(CShort(e.NewValue - e.OldValue))
      End Select

      If e.Type = ScrollEventType.LargeIncrement AndAlso UltraScrollBar1.Maximum <= e.NewValue + UltraScrollBar1.LargeChange Then
        UltraScrollBar1.Value = UltraScrollBar1.Maximum - UltraScrollBar1.LargeChange
      End If
      If e.Type <> ScrollEventType.ThumbTrack Then
        If e.NewValue = UltraScrollBar1.Minimum Then
          UltraScrollBar1.Minimum -= CInt(UltraScrollBar1.LargeChange / 2)
          UltraScrollBar1.Maximum -= CInt(UltraScrollBar1.LargeChange / 2)
        Else
          UltraChart1.Axis.X.RangeMin = startDateBefore.Ticks
          UltraChart1.Axis.X.RangeMax = startDateBefore.Ticks + spanTicks
        End If
        If e.NewValue + UltraScrollBar1.LargeChange >= UltraScrollBar1.Maximum Then
          UltraScrollBar1.Minimum += CInt(UltraScrollBar1.LargeChange / 2)
          UltraScrollBar1.Maximum += CInt(UltraScrollBar1.LargeChange / 2)
        End If
      End If
    End If
  End Sub
#End Region
#End Region

#Region "-- Event Methods --"
  'Insert code here
#End Region

#Region "-- Methods --"
  'Insert code here
#End Region

#Region "-- Private Subs --"
  Private Sub AddEvent(name As String)
    Try
      Dim item As New UltraListViewItem(CStr(ulvwEvents.Items.Count + 1))
      item.Value = ulvwEvents.Items.Count + 1
      ulvwEvents.Items.Add(item)
      item.SubItems.Item(EventListColumnKey.Time.ToString).Value = Now.ToShortTimeString
      item.SubItems.Item(EventListColumnKey.Name.ToString).Value = name
      item.BringIntoView()
    Catch ex As Exception
      Throw New Exception(String.Format("{0}.AddEvent Error", Me.GetType.Name), ex)
    End Try
  End Sub
  Private Sub CreateChartData()
    Try
      UltraChart1.Series.Clear()
      Dim initDate As DateTime = CDate("6 April 2010 9:00 am")
      For i As Integer = 1 To CInt(nudNumSeries.Value)
        Dim ganttSeries As GanttSeries = New GanttSeries(String.Format("{0}", i))
        For j As Integer = 1 To CInt(nudItemsPerSeries.Value)
          Dim ganttItem As GanttItem = ganttSeries.Items.Add(New GanttItem(String.Format("{0}.{1}", i, j)))
          Dim dummyTimeEntry As GanttTimeEntry = New GanttTimeEntry(Now.AddYears(-100), Now.AddYears(-100).AddHours(1), 0, -1, String.Format("{0}.{1}", i, j), 100)
          ganttItem.Times.Add(dummyTimeEntry)
          Dim fromDate As DateTime = initDate.AddHours(i + j)
          Dim toDate As DateTime = fromDate
          For k As Integer = 1 To CInt(nudTimesPerItem.Value)
            toDate = fromDate.AddHours(k)
            Dim sid As String = String.Concat(i.ToString("000"), j.ToString("000"), k.ToString("000"))
            Dim iid As Integer = Integer.Parse(sid)
            Dim timeEntry As GanttTimeEntry = New GanttTimeEntry(fromDate, toDate, iid, -1, String.Format("{0}.{1}", i, j), 100)
            'Dim timeEntry As GanttTimeEntry = New GanttTimeEntry(fromDate, toDate, Integer.Parse(String.Concat(i.ToString("000"), j.ToString("000"), k.ToString("000"))), -1, String.Format("{0}.{1}", i, j), 100)
            ganttItem.Times.Add(timeEntry)
            fromDate = toDate.AddHours(1)
          Next
        Next
        UltraChart1.Series.Add(ganttSeries)
      Next
      _CustomPrimitives.Clear()
      _CustomPrimitives.Add(New CustomPrimitive("First", 500, 50, 100, 20))
      _CustomPrimitives.Add(New CustomPrimitive("Second", 200, 100, 200, 20))
      _CustomPrimitives.Add(New CustomPrimitive("Thiid", 100, 150, 120, 20))
    Catch ex As Exception
      Throw New Exception(String.Format("{0}.CreateChartData Error", Me.GetType.Name), ex)
    End Try
  End Sub
  Private Sub SetupControls()
    Try
      ulvwPrimitives.ViewSettingsDetails.ImageSize = Size.Empty
      ulvwEvents.ViewSettingsDetails.ImageSize = Size.Empty
      UltraChart1.Axis.X.RangeMin = CDate("6 April 2010 9:00 am").Ticks
      UltraChart1.Axis.X.RangeMax = CDate("6 April 2010 9:00 am").AddHours(6).Ticks
      Dim labelHashTable As New Hashtable()
      labelHashTable.Add("GanttTimeEntryLabel", New GanttTimeEntryLabelRenderer(UltraChart1))
      UltraChart1.LabelHash = labelHashTable
      _ToolTipAppearance.BackColor = Color.MistyRose
      '_ToolTipAppearance.BackColor2 = result.BackColor

    Catch ex As Exception
      Throw New Exception(String.Format("{0}.SetupControls Error", Me.GetType.Name), ex)
    End Try
  End Sub
  Private Sub ShowItemToolTip(seriesKey As String, itemKey As String, timeEntryIndex As Integer)
    Try
      Dim toolTipInfo As UltraToolTipInfo = Me.UltraToolTipManager1.GetUltraToolTip(UltraChart1)
      If toolTipInfo Is Nothing Then
        toolTipInfo = New UltraToolTipInfo(String.Empty, ToolTipImage.Custom, itemKey, DefaultableBoolean.[True])
        Me.UltraToolTipManager1.SetUltraToolTip(UltraChart1, toolTipInfo)
      End If

      Dim sb As New System.Text.StringBuilder()
      sb.Append("<hr/>")

      Dim series As GanttSeries = CType(UltraChart1.Series.FromKey(seriesKey), GanttSeries)
      sb.AppendLine(String.Format("Series Key = '{0}'", series.Key))
      Dim item As GanttItem = series.Items.FromKey(itemKey)
      If item IsNot Nothing Then
        sb.AppendLine(String.Format("Item Key = '{0}'", item.Key))

        Dim timeEntry As GanttTimeEntry = item.Times.Item(timeEntryIndex)
        sb.AppendLine(String.Format("Item Key = '{0}'", timeEntry.ID))
        sb.AppendLine(String.Format("From = '{0}'", timeEntry.Start.ToString("d MMM yyyy hh:mm tt")))
        sb.AppendLine(String.Format("To   = '{0}'", timeEntry.End.ToString("d MMM yyyy hh:mm tt")))
      End If

      toolTipInfo.ToolTipTitle = itemKey
      toolTipInfo.Appearance = _ToolTipAppearance
      toolTipInfo.Appearance.Image = My.Resources.Employee
      toolTipInfo.ToolTipTextFormatted = sb.ToString().Replace(Environment.NewLine, "<br/>")
      toolTipInfo.Enabled = DefaultableBoolean.True

    Catch ex As Exception
      Throw New Exception(String.Format("{0}.ShowItemToolTip Error", Me.GetType.Name), ex)
    End Try
  End Sub
#End Region

End Class
