﻿#Region "-- Imports --"
Imports System.ComponentModel
Imports System.Drawing
Imports System.Text
Imports Infragistics.Shared
Imports Infragistics.Win
Imports Infragistics.Win.UltraWinGrid
Imports Infragistics.Win.UltraWinToolTip
Imports Infragistics.Win.UltraWinChart
Imports Infragistics.UltraChart.Core.Primitives
Imports Infragistics.UltraChart.Core
Imports Infragistics.UltraChart.Core.Layers
Imports Infragistics.UltraChart.Data
Imports Infragistics.UltraChart.Core.Util
Imports Infragistics.UltraChart.Resources.Appearance
Imports Infragistics.UltraChart.Shared.Styles
#End Region

''' <summary>
''' A class for drawing custom tooltips for <see cref="GanttTimeEntry"/> objects.
''' </summary>
Friend Class GanttTimeEntryLabelRenderer
  Implements Infragistics.UltraChart.Resources.IRenderLabel

  'Private _ChartData As GanttNodes = Nothing
  Private _GanttChart As UltraChart = Nothing

  'Since the labels will need information from the data, the constructor for this class takes a GanttDataSource. 
  Public Sub New(ByVal chart As UltraChart)
    '_ChartData = nodes
    _GanttChart = chart
  End Sub
  ''' <summary>
  ''' Take the custom label &lt;MyLabel&gt; and return the string to display in the control. 
  ''' </summary>
  ''' <param name="context"></param>
  Public Overloads Function ToString(ByVal context As System.Collections.Hashtable) As String Implements Infragistics.UltraChart.Resources.IRenderLabel.ToString
    ' NOTE : The ToString method will take the custom label <MyLabel> and return the string to display in the control. 
    'Debug.WriteLine(String.Format("GanttTimeEntryLabelRenderer.ToString : {0} {1} {2} {3}", "", "", "", ""))
    Dim sb As New System.Text.StringBuilder()
    If Not _GanttChart.Tooltips.Display = TooltipDisplay.Never Then
      Try
        'Dim isDummyTimeEntry As Boolean = False

        Dim timeEntryIndex As Integer = CInt(context("TIME_ENTRY_INDEX"))
        Dim seriesKey As String = CStr(context("SERIES_LABEL"))
        Dim itemKey As String = CStr(context("ITEM_LABEL"))

        If _GanttChart IsNot Nothing AndAlso _GanttChart.Series IsNot Nothing AndAlso seriesKey IsNot Nothing AndAlso seriesKey.Length > 0 Then
          Dim series As GanttSeries = CType(_GanttChart.Series.FromKey(seriesKey), GanttSeries)
          If series IsNot Nothing Then
            sb.AppendLine(String.Format("Series Key = '{0}'", series.Key))
            Dim item As GanttItem = series.Items.FromKey(itemKey)
            If item IsNot Nothing Then
              sb.AppendLine(String.Format("Item Key = '{0}'", item.Key))
            End If
            If item IsNot Nothing AndAlso item.Times.Count >= timeEntryIndex Then
              Dim timeEntry As GanttTimeEntry = item.Times(timeEntryIndex)
              sb.AppendLine(String.Format("Item Key = '{0}'", timeEntry.ID))
              sb.AppendLine(String.Format("From = '{0}'", timeEntry.Start.ToString("d MMM yyyy hh:mm tt")))
              sb.AppendLine(String.Format("To   = '{0}'", timeEntry.End.ToString("d MMM yyyy hh:mm tt")))
              'timeEntry.
            End If
          End If
        End If
      Catch ex As Exception
        sb.Append("GanttTimeEntryLabelRenderer.ToString Error")
        Debug.WriteLine(String.Format("GanttTimeEntryLabelRenderer.ToString Error : {0}", ex.InnerException.Message))
      End Try
    End If
    Return sb.ToString
  End Function

End Class
